from colorama import init, Fore, Back, Style
from os import system
from math import floor, ceil

init(autoreset=True)

def update_canvas(reason, controller):
    content = []
    for row in range(len(controller.content)):
        content.append([])
        for cell in range(len(controller.content[row])):
            if len(controller.content[row][cell]) > controller.column_widths[cell]-controller.settings['row_entry_margin']-len(controller.settings['pointer']):
                content[-1].append(controller.content[row][cell][:controller.column_widths[cell]-controller.settings['row_entry_margin']-len(controller.settings['pointer'])-1]+'…')
            else:
                content[-1].append(controller.content[row][cell])

    if controller.model['model_id'] == 'FramedList':
        ready_row = ''
        system('cls')
        print(controller.model['first_row'] + '\n' + controller.model['second_row'] + '\n' + controller.model['third_row'])
        if controller.position[1] > len(content)-1:
            controller.position[1] = len(content)-1
        elif controller.position[1] < 0:
            controller.position[1] = 0

        if len(content) > controller.canvas_height-4:
            if controller.position[1] > controller.view_anchor + floor(controller.canvas_height-4-controller.settings['list_scroll_margin']-1):
                controller.view_anchor += 1
            elif controller.position[1] < controller.view_anchor + controller.settings['list_scroll_margin'] and controller.view_anchor > 0:
                controller.view_anchor -= 1
        else:
            controller.view_anchor = 0

        for row in range(controller.view_anchor, controller.view_anchor+floor(controller.canvas_height-4)):
            ready_row = controller.settings['margin_left']*' '
            if row == controller.position[1]:
                ready_row += controller.model['divider'] + controller.settings['pointer'] + Back.WHITE + Fore.BLACK
                for cell in range(len(content[row])):
                    ready_row += ' '*controller.settings['row_entry_margin'] + content[row][cell] + ' '*(controller.column_widths[cell]-len(content[row][cell])+1-controller.settings['row_entry_margin']-(len(controller.settings['pointer']) if cell == 0 else 0))
                ready_row = ready_row[:-1] + Back.RESET + Fore.RESET + controller.model['divider']
            elif row < len(content):
                ready_row += controller.model['divider']
                for cell in range(len(content[row])):
                    ready_row += ' '*(controller.settings['row_entry_margin']+(len(controller.settings['pointer']) if cell == 0 else 0)) + content[row][cell] + ' '*(controller.column_widths[cell]-len(content[row][cell])-controller.settings['row_entry_margin']-(len(controller.settings['pointer']) if cell == 0 else 0)) + controller.model['divider']
            else:
                ready_row += controller.model['divider']
                for cell in range(len(controller.column_widths)):
                    ready_row += ' '*((controller.column_widths[cell])) + controller.model['divider']
            print(ready_row)
        last_row, footer = controller.settings['margin_left']*' ' + '┗', ''
        for column in range(len(controller.column_widths)):
            last_row += '━'*controller.column_widths[column] + '┻'

        if len(controller.settings['bottom_footer']) > 0:
            if len(controller.settings['bottom_footer']) > controller.settings['margin_bottom']:
                footer = '\n' + '\n'.join(controller.settings['bottom_footer'][:controller.settings['margin_bottom']])
            else:
                footer = '\n' + '\n'.join(controller.settings['bottom_footer']) + (controller.settings['margin_bottom']-len(controller.settings['bottom_footer']))*'\n'
        else:
            footer = controller.settings['margin_bottom']*'\n'

        print(last_row[:-1] + '┛' + controller.settings['margin_right']*' ' + footer, end='\r')


def calculate_widths(controller, new_column=None):
    columns, raw_columns = [], []
    for i in controller.columns:
        raw_columns.append(i)
    total_autos, total_manuals = 0, 0
    if new_column != None:
        raw_columns.append(new_column)
    for column in range(len(raw_columns)):
        if raw_columns[column][1] == 'auto':
            total_autos += 1
        else:
            total_manuals += raw_columns[column][1]
    for column in range(len(raw_columns)):
        if raw_columns[column][1] == 'auto':
            columns.append(int((controller.canvas_width-total_manuals-len(raw_columns)+1)/total_autos))
        else:
            columns.append(raw_columns[column][1])
    return columns

def get_model(controller, model_type):
    columns = []
    column_sizes = calculate_widths(controller)
    first_row, second_row, third_row = controller.settings['margin_left']*' ' + '┏', controller.settings['margin_left']*' ' + '┃', controller.settings['margin_left']*' ' + '┣'
    for i in range(len(controller.columns)):
        columns.append([controller.columns[i][0], column_sizes[i]])
        first_row += '━'*columns[i][1] + '┳'
        second_row += ((' '*controller.settings['column_label_margin'] + columns[i][0] + ' '*(columns[i][1]-len(columns[i][0])-controller.settings['column_label_margin'])) if len(columns[i][0])+controller.settings['column_label_margin'] <= columns[i][1] else (columns[i][0][:columns[i][1]-1-controller.settings['column_label_margin']] + '…')) + '┃'
        third_row += '━'*columns[i][1] + '╋'
    first_row, third_row = first_row[:-1] + '┓', third_row[:-1] + '┫'
    if len(controller.settings['top_header']) > 0:
        if len(controller.settings['top_header']) > controller.settings['margin_top']:
            header = '\n'.join(controller.settings['top_header'][:controller.settings['margin_top']])
        else:
            header = '\n'.join(controller.settings['top_header']) + (controller.settings['margin_top']-len(controller.settings['top_header']))*'\n'
    else:
        header = controller.settings['margin_top']*'\n'
    return {'model_id': 'FramedList', 'first_row': header + '\n' + first_row + controller.settings['margin_right']*' ', 'second_row': second_row + controller.settings['margin_right']*' ', 'third_row': third_row + controller.settings['margin_right']*' ', 'divider': '┃', 'columns': columns}